# -*- coding: utf-8 -*-


class Comparable(object):

    def __init__(self, foo='bar'):
        self.foo = foo

    def __hash__(self):
        return hash(self.foo)

    def __lt__(self, other):
        return self.foo < other.foo

    def __le__(self, other):
        return self.foo <= other.foo

    def __eq__(self, other):
        return self.foo == other.foo

    def __ne__(self, other):
        return self.foo != other.foo

    def __gt__(self, other):
        return self.foo > other.foo

    def __ge__(self, other):
        return self.foo >= other.foo

    def __cmp__(self, other):
        return cmp(self.foo, other.foo)


class AnotherComparable(object):

    def __init__(self, baz='qux'):
        self.baz = baz

    def __hash__(self):
        return hash(self.baz)

    def __lt__(self, other):
        return self.baz < other.baz

    def __le__(self, other):
        return self.baz <= other.baz

    def __eq__(self, other):
        return self.baz == other.baz

    def __ne__(self, other):
        return self.baz != other.baz

    def __gt__(self, other):
        return self.baz > other.baz

    def __ge__(self, other):
        return self.baz >= other.baz

    def __cmp__(self, other):
        return cmp(self.baz, other.baz)



# -*- coding: utf-8 -*-
'''preggy core: the `Expect` class, and the `@assertion` and `@create_assertions` decorators.
'''
# preggy assertions
# https://github.com/heynemann/preggy

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2013 Bernardo Heynemann heynemann@gmail.com

from __future__ import absolute_import
import functools

from preggy import utils

_registered_assertions = utils.AssertionsMap()


def assertion(func):
    '''Function decorator.  Provides lower-level control for custom
    assertions than `@preggy.create_assertions`.

    This decorator is preferable over `@preggy.create_assertions`
    if you need to fine-tune your error message, or if your assertion
    doesn’t have a corresponding `not_`.

    Unlike `@preggy.create_assertions`, functions decorated with
    this shouldn’t return a boolean. Instead, they should check for
    undesirable conditions and raise an `AssertionError` when appropriate.

    Whenever possible, you should declare both the normal assertion as well
    as a `not_` counterpart, so they can be used like this:

    ... # doctest: +ELLIPSIS, +NORMALIZE_WHITESPACE
    >>>
    >>> import preggy
    >>> from preggy.core import Expect as expect
    >>>
    >>> @preggy.assertion
    ... def to_be_a_positive_integer(topic):
    ...     if not topic > 0:
    ...          raise AssertionError("Expected topic('{topic}') to be a positive integer".format(topic=topic))
    ...
    >>> @preggy.assertion
    ... def not_to_be_a_positive_integer(topic):
    ...     if not topic < 0:
    ...          raise AssertionError("Expected topic('{topic}') not to be a positive integer".format(topic=topic))
    ...
    >>> expect(5).to_be_a_positive_integer()
    >>> expect(-3).Not.to_be_a_positive_integer()

    '''
    if not hasattr(func, 'humanized'):
        setattr(func, 'humanized', utils.humanized_name(func.__name__))

    @functools.wraps(func)
    def wrapper(*args, **kw):
        func(*args, **kw)
        return Expect(args[0])

    _registered_assertions[wrapper.__name__] = wrapper
    return wrapper


def create_assertions(func):
    '''Function decorator.  Use to create custom assertions for your
    tests.
    ''' '''
    Creating new assertions for use with `expect` is as simple as using
    this decorator on a function. The function expects `topic` as the
    first parameter, and `expectation` second:

        @preggy.create_assertions
        def to_be_greater_than(topic, expected):
            return topic > expected

    This creates both the assertion AND its `not_*` counterpart.

    ... # doctest: +ELLIPSIS, +NORMALIZE_WHITESPACE
    >>> from preggy.core import (assertion, create_assertions, Expect as expect)
    >>> from preggy.assertions import *

    >>> expect(2).to_be_greater_than(3)
    Traceback (most recent call last):
        ...
    AssertionError: Expected topic(2) to be greater than 3

    >>> expect(4).not_to_be_greater_than(3)
    Traceback (most recent call last):
        ...
    AssertionError: Expected topic(4) not to be greater than 3

    '''
    # set custom func attribute "humanized"
    setattr(func, 'humanized', utils.humanized_name(func.__name__))

    # modified functools.update_wrapper
    def _update_wrapper(wrapper, wrapped, not_assertion=True):
        '''A modified version of functools.update_wrapper. Auto-modifies the
        wrapper's __name__ and __doc__ to create a not_assertion.
        '''
        # begin as usual
        wrapper = functools.update_wrapper(wrapper, wrapped)

        # compute overrides for not_* assertions values
        if not_assertion:
            new_name = 'not_{0.__name__}'.format(wrapped)
            new_doc = ''  # .format(wrapped)

            # set our overrides
            setattr(wrapper, '__name__', new_name)
            setattr(wrapper, '__doc__', new_doc)
            # update to reflect new __name__
            setattr(wrapper, 'humanized', utils.humanized_name(wrapper.__name__))

        # Return the wrapper so this can be used as a decorator via partial()
        return wrapper

    # First assertion
    @assertion
    @functools.wraps(func)
    def test_assertion(*args):
        if not func(*args):
            raw_msg = utils.format_assertion_msg(func.humanized, *args)
            err_msg = raw_msg.format(*args)
            raise AssertionError(err_msg)
        return Expect(args[0])

    # Second assertion: prepare
    def test_not_assertion(*args):
        if func(*args):
            raw_msg = utils.format_assertion_msg('not {0!s}'.format(func.humanized), *args)
            err_msg = raw_msg.format(*args)
            raise AssertionError(err_msg)
        return Expect(args[0])

    # Second assertion: update and register
    test_not_assertion = _update_wrapper(test_not_assertion, func)
    test_not_assertion = assertion(test_not_assertion)


class ErrorToHappenContext(object):
    def __init__(self, error_class, message=None):
        self.error_class = error_class
        self.message = message
        self._error = None

    @property
    def error(self):
        return self._error

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        has_exception = exc_type is not None
        is_subclass = has_exception and (exc_type is self.error_class or issubclass(exc_type, self.error_class)) or False
        if not has_exception:
            raise AssertionError('Expected "%s.%s" to happen but no errors happened during execution of with block.' % (
                self.error_class.__module__,
                self.error_class.__name__,
            ))

        if has_exception and not is_subclass:
            raise AssertionError('Expected "%s.%s" to happen but "%s.%s" happened during execution of with block.' % (
                self.error_class.__module__,
                self.error_class.__name__,
                exc_type.__module__,
                exc_type.__name__
            ))

        if self.message is not None:
            error_msg = getattr(exc_val, 'message', utils.text_type(exc_val))
            if error_msg != self.message:
                raise AssertionError('Expected "%s.%s" to have a message of "%s", but the actual error was "%s".' % (
                    self.error_class.__module__,
                    self.error_class.__name__,
                    self.message,
                    error_msg
                ))

        if has_exception and is_subclass:
            self._error = exc_val
            return True

        return False


class NotErrorToHappenContext(object):
    def __init__(self, error_class, message=None):
        self.error_class = error_class
        self.message = message
        self._error = None

    @property
    def error(self):
        return self._error

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        has_exception = exc_type is not None
        is_subclass = has_exception and (exc_type is self.error_class or issubclass(exc_type, self.error_class)) or False

        if has_exception and self.message is not None:
            error_msg = getattr(exc_val, 'message', utils.text_type(exc_val))
            if error_msg != self.message:
                raise AssertionError('Expected "%s.%s" to have a message of "%s", but the actual error was "%s".' % (
                    self.error_class.__module__,
                    self.error_class.__name__,
                    self.message,
                    error_msg
                ))

        if has_exception and not is_subclass:
            raise AssertionError('Expected "%s.%s" not to happen but "%s.%s" happened during execution of with block.' % (
                self.error_class.__module__,
                self.error_class.__name__,
                exc_type.__module__,
                exc_type.__name__
            ))

        if has_exception:
            raise AssertionError('Expected "%s.%s" not to happen but it happened during execution of with block.' % (
                self.error_class.__module__,
                self.error_class.__name__,
            ))


class Expect(object):
    '''This atypical class provides a key part of the preggy testing syntax.

    For example:

        >>> from preggy.core import (assertion, create_assertions, Expect as expect)
        >>> from preggy.assertions import *
        >>>
        >>> expect(True).to_be_true()
        >>> expect(False).to_be_false()

    '''

    def __init__(self, topic):
        self.topic = topic
        if isinstance(self.topic, ErrorToHappenContext):
            self.topic = self.topic.error

        self.not_assert = False

    @classmethod
    def not_to_be_here(cls, message=None):
        error = ''
        if message is not None:
            error = ' (%s)' % message

        raise AssertionError("Should not have gotten this far%s." % error)

    @classmethod
    def error_to_happen(cls, error_class=Exception, message=None):
        return ErrorToHappenContext(error_class, message=message)

    @classmethod
    def not_error_to_happen(cls, error_class=Exception, message=None):
        return NotErrorToHappenContext(error_class, message=message)

    @classmethod
    def error_not_to_happen(cls, error_class=Exception, message=None):
        return cls.not_error_to_happen(error_class, message)

    def __getattr__(self, name):
        # common cases
        if name in ['topic', 'not_to_be_here']:
            return super(Expect, self).__getattr__(name)
        if name == 'Not':
            self.not_assert = not self.not_assert
            return self

        # determine whether assertion is of "not" form
        method_name = 'not_{name}'.format(name=name) if self.not_assert else name

        # if program gets this far, then it’s time to perform the assertion. (...FINALLY! ;D)
        def _assert_topic(*args, **kw):
            # Allows chained calls to assertions, such as `expect(topic).to_be_true()`.
            return _registered_assertions[method_name](self.topic, *args, **kw)

        return _assert_topic



# -*- coding: utf-8 -*-

# preggy assertions
# https://github.com/heynemann/preggy

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2013 Bernardo Heynemann heynemann@gmail.com

from preggy import expect

#-----------------------------------------------------------------------------

EXPECTED_TEST_DATA = (
    True,
    'qwe',
    1,
    [1],
    tuple([1]),
    {'a': 'b'}
)

NOT_EXPECTED_TEST_DATA = (
    False,
    '',
    0,
    [],
    tuple([]),
    {}
)

#-----------------------------------------------------------------------------

def is_true(item):
    expect(item).to_be_true()
    try:
        expect(item).not_to_be_true()
    except AssertionError:
        return
    assert False, 'Should not have gotten this far'


def is_not_true(item):
    expect(item).Not.to_be_true()
    expect(item).not_to_be_true()

    try:
        expect(item).to_be_true()
    except AssertionError:
        return
    assert False, 'Should not have gotten this far'


def is_false(item):
    expect(item).to_be_false()
    try:
        expect(item).not_to_be_false()
    except AssertionError:
        return
    assert False, 'Should not have gotten this far'


def is_not_false(item):
    expect(item).Not.to_be_false()
    expect(item).not_to_be_false()

    try:
        expect(item).to_be_false()
    except AssertionError:
        return
    assert False, 'Should not have gotten this far'

#-----------------------------------------------------------------------------

def test_to_be_true():
    for item in EXPECTED_TEST_DATA:
        yield is_true, item


def test_not_to_be_true():
    for item in NOT_EXPECTED_TEST_DATA:
        yield is_not_true, item


def test_to_be_false():
    for item in NOT_EXPECTED_TEST_DATA:
        yield is_false, item


def test_not_to_be_false():
    for item in EXPECTED_TEST_DATA:
        yield is_not_false, item



# -*- coding: utf-8 -*-

# preggy assertions
# https://github.com/heynemann/preggy

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2013 Bernardo Heynemann heynemann@gmail.com

import sys
try:
    from Queue import LifoQueue
except ImportError:
    from queue import LifoQueue

from preggy import expect

#-----------------------------------------------------------------------------

queue = LifoQueue()
queue.put(1)
queue.put(2)
queue.put(3)

TEST_DATA = (
    '12345',
    [1, 2, 3],
    {'a': 1, 'b': 2},
    tuple([1, 2, 3, 4, 5, 6]),
    queue
)

EXPECTED_DATA = (
    5,
    3,
    2,
    6,
    3
)

NOT_EXPECTED_DATA = (
    7,
    2,
    3,
    5,
    5
)

#-----------------------------------------------------------------------------

def is_expected(item, expected):
    expect(item).to_length(expected)
    try:
        expect(item).not_to_length(expected)
    except AssertionError:
        return
    assert False, 'Should not have gotten this far'


def is_not_expected(item, expected):
    expect(item).Not.to_length(expected)
    expect(item).not_to_length(expected)

    try:
        expect(item).to_length(expected)
    except AssertionError:
        return
    assert False, 'Should not have gotten this far'

#-----------------------------------------------------------------------------

def test_length():
    for index, item in enumerate(TEST_DATA):
        yield is_expected, item, EXPECTED_DATA[index]


def test_not_includes():
    for index, item in enumerate(TEST_DATA):
        yield is_not_expected, item, NOT_EXPECTED_DATA[index]


def test_unable_to_identify_length():
    try:
        expect(object()).to_length(1)
    except AssertionError:
        exc = sys.exc_info()[1]
        expect(str(exc)).to_match('Could not determine "<object object at .+"\'s length.')
        return

    assert False, 'Should not have gotten this far'


def test_unable_to_identify_not_length():
    try:
        expect(object()).not_to_length(1)
    except AssertionError:
        exc = sys.exc_info()[1]
        expect(str(exc)).to_match('Could not determine "<object object at .+"\'s length.')
        return

    assert False, 'Should not have gotten this far'



# -*- coding: utf-8 -*-
'''preggy "null" (None) assertions.  For use with `expect()` (see `preggy.core`).
'''

# preggy assertions
# https://github.com/heynemann/preggy

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2013 Bernardo Heynemann heynemann@gmail.com

from __future__ import absolute_import
from preggy import assertion


@assertion
def to_be_null(topic):
    '''Asserts that `topic` is `None`.'''
    if topic is not None:
        msg = 'Expected topic({0}) to be None'.format(topic)
        raise AssertionError(msg)


@assertion
def not_to_be_null(topic):
    '''Asserts that `topic` is NOT `None`.'''
    if topic is None:
        msg = 'Expected topic({0}) not to be None'.format(topic)
        raise AssertionError(msg)



# -*- coding: utf-8 -*-

# preggy assertions
# https://github.com/heynemann/preggy

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2013 Bernardo Heynemann heynemann@gmail.com

from preggy import expect

from tests import Comparable, AnotherComparable

#-----------------------------------------------------------------------------

TEST_DATA = (
    'qwe',
    b'qwe',
    b'\xff\xd8\xff\xe0\x00\x10JFIF',
    [1],
    {'a': 'b'},
    tuple([2]),
    Comparable()
)

UNICODE_TEST_DATA = (
    b'asdqwe123',
    'asdqwe123',
)

UNEQUAL_DATA = (
    'asd',
    [2],
    {'c': 'd'},
    tuple([3]),
    Comparable('baz'),
    AnotherComparable()
)

#-----------------------------------------------------------------------------


def is_equal(topic):
    item, expected = topic
    expect(item).to_equal(expected)


def is_not_equal(topic):
    item, expected = topic
    expect(item).Not.to_equal(expected)
    expect(item).not_to_equal(expected)


#-----------------------------------------------------------------------------


def test_unicode_equal():
    is_equal((UNICODE_TEST_DATA[0], UNICODE_TEST_DATA[1]))


def test_equal():
    for item in TEST_DATA:
        yield is_equal, (item, item)


def test_not_equal():
    for item in TEST_DATA:
        for unequal in UNEQUAL_DATA:
            yield is_not_equal, (item, unequal)



# -*- coding: utf-8 -*-

# preggy assertions
# https://github.com/heynemann/preggy

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2013 Bernardo Heynemann heynemann@gmail.com

from preggy import expect

from tests import Comparable

#-----------------------------------------------------------------------------

class FakeClass(object): pass
class Other(FakeClass):  pass
class Another(FakeClass, Comparable): pass

TEST_DATA = frozenset([
    FakeClass,
    FakeClass(),
    Other,
    Other(),
    Another,
    Another()
])

#-----------------------------------------------------------------------------

def is_expected(item):
    expect(item).to_be_instance_of(FakeClass)
    try:
        expect(item).not_to_be_instance_of(FakeClass)
    except AssertionError:
        return
    assert False, 'Should not have gotten this far'


def is_not_expected(item):
    expect(item).Not.to_be_instance_of(dict)
    expect(item).not_to_be_instance_of(dict)

    try:
        expect(item).to_be_instance_of(dict)
    except AssertionError:
        return
    assert False, 'Should not have gotten this far'

#-----------------------------------------------------------------------------

def test_to_be_instance_of():
    for item in TEST_DATA:
        yield is_expected, item


def test_not_to_be_instance_of():
    for item in TEST_DATA:
        yield is_not_expected, item



# -*- coding: utf-8 -*-

# preggy assertions
# https://github.com/heynemann/preggy

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2013 Bernardo Heynemann heynemann@gmail.com

from preggy import expect

#-----------------------------------------------------------------------------

TEST_DATA = frozenset([
    './README.md',
])

FAIL_DATA = frozenset([
    './READM.md',
    0,
    (1, 2, 3)
])

#-----------------------------------------------------------------------------

def is_expected(item):
    with open(item) as f:
        expect(f).to_be_a_file()
        expect(item).to_be_a_file()
        try:
            expect(item).not_to_be_a_file()
        except AssertionError:
            return
        assert False, 'Should not have gotten this far'


def is_not_expected(item):
    expect(item).Not.to_be_a_file()
    expect(item).not_to_be_a_file()

    try:
        expect(item).to_be_a_file()
    except AssertionError:
        return

    assert False, 'Should not have gotten this far'

#-----------------------------------------------------------------------------

def test_to_be_a_file():
    for item in TEST_DATA:
        yield is_expected, item


def test_not_to_be_a_file():
    for item in FAIL_DATA:
        yield is_not_expected, item


def test_is_not_file_obj():
    with open('./README.md') as f:
        try:
            expect(f).not_to_be_a_file()
        except AssertionError:
            return

        assert False, 'Should not have gotten this far'



# -*- coding: utf-8 -*-

# preggy assertions
# https://github.com/heynemann/preggy

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2013 Bernardo Heynemann heynemann@gmail.com

from preggy import expect

#-----------------------------------------------------------------------------

TEST_DATA = (
    'qwe',
    b'qwe',
    5,
    {'a': 'b', 'c': 'd'},
    tuple([2, 3]),
    list([2, 3]),
    set([2, 3]),
)

GREATER_THAN_DATA = (
    'asd',
    b'asd',
    2,
    {'c': 'd'},
    tuple([3]),
    list([3]),
    set([3]),
)

LESSER_THAN_DATA = (
    'zcs',
    b'zcs',
    7,
    {'a': 'b', 'c': 'd', 'e': 'f'},
    tuple([3, 4, 5]),
    list([3, 4, 5]),
    set([3, 4, 5]),
)

LESSER_OR_EQUAL_TO_DATA = (
    'qwe',
    b'qwe',
    5,
    {'a': 'b', 'c': 'd'},
    tuple([2, 3]),
    list([2, 3]),
    set([2, 3]),
)

NOT_LESSER_OR_EQUAL_TO_DATA = (
    'zcs',
    b'zcs',
    7,
    {'a': 'b', 'c': 'd', 'e': 'f'},
    tuple([3, 4, 5]),
    list([3, 4, 5]),
    set([3, 4, 5]),
)

LESSER_OR_EQUAL_TO_DATA_2 = (
    'zcs',
    b'zcs',
    7,
    {'a': 'b', 'c': 'd', 'e': 'f'},
    tuple([3, 4, 5]),
    list([3, 4, 5]),
    set([3, 4, 5]),
)

GREATER_OR_EQUAL_TO_DATA = (
    'qwe',
    b'qwe',
    5,
    {'a': 'b', 'c': 'd'},
    tuple([2, 3]),
    list([2, 3]),
    set([2, 3]),
)

NOT_GREATER_OR_EQUAL_TO_DATA = (
    'abc',
    b'abc',
    1,
    {'a': 'b'},
    tuple([3]),
    list([3]),
    set([3]),
)

GREATER_OR_EQUAL_TO_DATA_2 = (
    'abc',
    b'abc',
    1,
    {'a': 'b'},
    tuple([3]),
    list([3]),
    set([3]),
)


#-----------------------------------------------------------------------------


def is_greater_than(topic):
    item, expected = topic
    expect(item).to_be_greater_than(expected)


def is_not_greater_than(topic):
    item, expected = topic
    expect(item).Not.to_be_greater_than(expected)
    expect(item).not_to_be_greater_than(expected)


def is_lesser_than(topic):
    item, expected = topic
    expect(item).to_be_lesser_than(expected)


def is_not_lesser_than(topic):
    item, expected = topic
    expect(item).Not.to_be_lesser_than(expected)
    expect(item).not_to_be_lesser_than(expected)


def is_greater_or_equal_to(topic):
    item, expected = topic
    expect(item).to_be_greater_or_equal_to(expected)


def is_not_greater_or_equal_to(topic):
    item, expected = topic
    expect(item).Not.to_be_greater_or_equal_to(expected)
    expect(item).not_to_be_greater_or_equal_to(expected)


def is_lesser_or_equal_to(topic):
    item, expected = topic
    expect(item).to_be_lesser_or_equal_to(expected)


def is_not_lesser_or_equal_to(topic):
    item, expected = topic
    expect(item).Not.to_be_lesser_or_equal_to(expected)
    expect(item).not_to_be_lesser_or_equal_to(expected)


#-----------------------------------------------------------------------------


def test_greater_than():
    for index, item in enumerate(TEST_DATA):
        expected = GREATER_THAN_DATA[index]
        yield is_greater_than, (item, expected)


def test_not_greater_than():
    for index, item in enumerate(TEST_DATA):
        expected = GREATER_THAN_DATA[index]
        yield is_not_greater_than, (expected, item)


def test_lesser_than():
    for index, item in enumerate(TEST_DATA):
        expected = LESSER_THAN_DATA[index]
        yield is_lesser_than, (item, expected)


def test_not_lesser_than():
    for index, item in enumerate(TEST_DATA):
        expected = LESSER_THAN_DATA[index]
        yield is_not_lesser_than, (expected, item)


def test_greater_or_equal_to():
    for index, item in enumerate(TEST_DATA):
        expected = GREATER_OR_EQUAL_TO_DATA[index]
        yield is_greater_or_equal_to, (item, expected)

    for index, item in enumerate(TEST_DATA):
        expected = GREATER_OR_EQUAL_TO_DATA_2[index]
        yield is_greater_or_equal_to, (item, expected)


def test_not_greater_or_equal_to():
    for index, item in enumerate(TEST_DATA):
        expected = NOT_GREATER_OR_EQUAL_TO_DATA[index]
        yield is_not_greater_or_equal_to, (expected, item)

    for index, item in enumerate(TEST_DATA):
        expected = GREATER_OR_EQUAL_TO_DATA_2[index]
        yield is_not_greater_or_equal_to, (expected, item)


def test_lesser_or_equal_to():
    for index, item in enumerate(TEST_DATA):
        expected = LESSER_OR_EQUAL_TO_DATA[index]
        yield is_lesser_or_equal_to, (item, expected)

    for index, item in enumerate(TEST_DATA):
        expected = LESSER_OR_EQUAL_TO_DATA_2[index]
        yield is_lesser_or_equal_to, (item, expected)


def test_not_lesser_or_equal_to():
    for index, item in enumerate(TEST_DATA):
        expected = NOT_LESSER_OR_EQUAL_TO_DATA[index]
        yield is_not_lesser_or_equal_to, (expected, item)

    for index, item in enumerate(TEST_DATA):
        expected = LESSER_OR_EQUAL_TO_DATA_2[index]
        yield is_not_lesser_or_equal_to, (expected, item)



# -*- coding: utf-8 -*-
'''preggy inclusion assertion.  For use with `expect()` (see `preggy.core`).
'''


# preggy assertions
# https://github.com/heynemann/preggy

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2013 Bernardo Heynemann heynemann@gmail.com

from __future__ import absolute_import

try:
    import six
except ImportError:  # pragma: no cover
    import warnings
    warnings.warn("Ignoring six. Probably setup.py installing package.")

from preggy import create_assertions


@create_assertions
def to_include(topic, expected):
    '''Asserts that `expected` is in `topic`.'''
    if isinstance(topic, six.string_types + tuple([six.binary_type, six.text_type])):
        return str(expected) in topic
    for t in topic:
        try:
            if expected == t:
                return True
        except:
            pass
    return False



# -*- coding: utf-8 -*-

# preggy assertions
# https://github.com/heynemann/preggy

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2013 Bernardo Heynemann heynemann@gmail.com

from preggy import expect

#-----------------------------------------------------------------------------

EMPTY_DATA = (
    '',
    [],
    {},
    tuple([])
)

NOT_EMPTY_DATA = (
    'qwe',
    [1],
    {'a': 'b'},
    tuple([2])
)

#-----------------------------------------------------------------------------

def is_empty(item):
    expect(item).to_be_empty()


def is_not_empty(item):
    expect(item).Not.to_be_empty()
    expect(item).not_to_be_empty()

#-----------------------------------------------------------------------------

def test_emptiness_assertion_works():
    for empty_item in EMPTY_DATA:
        yield is_empty, empty_item


def test_not_emptiness_assertion_works():
    for not_empty_item in NOT_EMPTY_DATA:
        yield is_not_empty, not_empty_item



# -*- coding: utf-8 -*-
'''preggy file assertions.  For use with `expect()` (see `preggy.core`).

For these assertions, "file" can be either a string or a file object.  Since
a string is required to create a file object in the first place, these
assertions provide a convenient, flexible way to test whether a topic is
a "file" in your tests.

'''
# preggy assertions
# https://github.com/heynemann/preggy

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2013 Bernardo Heynemann heynemann@gmail.com

from __future__ import absolute_import

try:
    import six
except ImportError:  # pragma: no cover
    import warnings
    warnings.warn("Ignoring six. Probably setup.py installing package.")

try:
    import io
except ImportError:  # pragma: no cover
    pass  ## FIXME: explain using "pass" here

from os.path import isfile
import types

from preggy import assertion


#-------------------------------------------------------------------------------------------------
# Helpers
#-------------------------------------------------------------------------------------------------
_is_file = lambda topic: isfile(topic)
_is_string = lambda topic: isinstance(topic, (six.string_types, six.text_type))

def _is_file_obj(topic):
    try:
        return isinstance(topic, types.FileType)
    except AttributeError:  # pragma: no cover
        # FIXME: add comment...
        #        what is this for?
        return isinstance(topic, io.IOBase)


#-------------------------------------------------------------------------------------------------
# Assertions
#-------------------------------------------------------------------------------------------------
@assertion
def to_be_a_file(topic):
    '''Asserts that `topic` is a file.

    If `topic` is a string, this asserts that `os.path.isfile()`
    returns `True`.

    Otherwise, this asserts whether `topic` is an instance of the
    built-in `file` type.

    '''
    if _is_string(topic) and _is_file(topic):
        return True
    elif _is_file_obj(topic):
        return True
    msg = 'Expected topic({0}) to be a file, but it was {1!r}'.format(topic, type(topic))
    raise AssertionError(msg)

@assertion
def not_to_be_a_file(topic):
    '''Asserts that `topic` is NOT a file.

    If `topic` is a string, this asserts that `os.path.isfile()`
    returns `False`.

    Otherwise, this asserts whether `topic` is NOT an instance of the
    built-in `file` type.

    '''
    try:
        to_be_a_file(topic)
    except AssertionError:
        return True
    msg = 'Expected topic({0}) not to be a file, but it was'.format(topic)
    raise AssertionError(msg)
    


# -*- coding: utf-8 -*-
'''preggy instance assertions.  For use with `expect()` (see `preggy.core`).
'''

# preggy assertions
# https://github.com/heynemann/preggy

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2013 Bernardo Heynemann heynemann@gmail.com

from __future__ import absolute_import
import inspect

from preggy import assertion


@assertion
def to_be_instance_of(topic, expected):
    '''Asserts that `topic` is an instance of `expected`.'''
    TRUE_CONDITIONS = [
        isinstance(topic, expected),
        (inspect.isclass(topic) and inspect.isclass(expected)) and issubclass(topic, expected),
    ]
    try:
        TRUE_CONDITIONS.append(topic == expected)
    except:
        pass
    if any(TRUE_CONDITIONS):
        return True
    msg = 'Expected topic({0}) to be an instance of {1}, but it was a {2}'.format(
        topic,
        expected,
        topic.__class__)
    raise AssertionError(msg)


@assertion
def not_to_be_instance_of(topic, expected):
    '''Asserts that `topic` is NOT an instance of `expected`.'''
    try:
        to_be_instance_of(topic, expected)
    except AssertionError:
        return True
    msg = 'Expected topic({0}) not to be an instance of {1}'.format(topic, expected)
    raise AssertionError(msg)



# -*- coding: utf-8 -*-
from __future__ import absolute_import

from preggy.assertions.types.boolean import *
from preggy.assertions.types.classes import *
from preggy.assertions.types.errors import *
from preggy.assertions.types.file import *
from preggy.assertions.types.function import *
from preggy.assertions.types.nullable import *
from preggy.assertions.types.numeric import *
from preggy.assertions.types.regexp import *



# -*- coding: utf-8 -*-

# preggy assertions
# https://github.com/heynemann/preggy

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2013 Bernardo Heynemann heynemann@gmail.com

from preggy import expect

#-----------------------------------------------------------------------------

def test_to_be_null():
    expect(None).to_be_null()

    try:
        expect(None).not_to_be_null()
    except AssertionError:
        return

    assert False, 'Should not have gotten this far'


def test_not_to_be_null():
    expect('something').Not.to_be_null()
    expect('something').not_to_be_null()

    try:
        expect('something').to_be_null()
    except AssertionError:
        return

    assert False, 'Should not have gotten this far'



# -*- coding: utf-8 -*-
'''preggy error assertions.  For use with `expect()` (see `preggy.core`).
'''

# preggy assertions
# https://github.com/heynemann/preggy

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2013 Bernardo Heynemann heynemann@gmail.com

from __future__ import absolute_import
import inspect

from preggy import assertion, create_assertions, utils


@assertion
def to_be_an_error_like(topic, expected):
    '''Asserts that `topic` is an instance (or subclass) of type `expected`.'''
    if not isinstance(topic, expected):
        msg = 'Expected topic({0}) to be an error of type {1}, but it was a {2}'
        values = topic, expected, topic.__class__
        err = AssertionError(msg.format(*values), *values)
        raise err


@assertion
def to_have_an_error_message_of(topic, expected):
    '''Asserts that `topic` has an error message of `expected`.'''
    if utils.text_type(topic) != expected:
        msg = 'Expected topic({0!r}) to be an error with message {1!r}'
        values = utils.text_type(topic), expected
        err = AssertionError(msg.format(*values))
        raise err


@create_assertions
def to_be_an_error(topic):
    '''Asserts that `topic` is an error.'''
    return isinstance(topic, BaseException)


@assertion
def not_to_be_an_error_like(topic, expected):
    '''Asserts that `topic` not is an instance (or subclass) of type `expected`.'''
    if isinstance(topic, expected):
        msg = 'Expected topic({0}) not to be an error of type {1}, but it was a {2}'
        values = topic, expected, topic.__class__
        err = AssertionError(msg.format(*values), *values)
        raise err



# -*- coding: utf-8 -*-
'''preggy regular expression assertions.  For use with `expect()` (see `preggy.core`).
'''

# preggy assertions
# https://github.com/heynemann/preggy

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2013 Bernardo Heynemann heynemann@gmail.com

from __future__ import absolute_import
import re

from preggy import assertion


@assertion
def to_match(topic, expected):
    '''Asserts that `topic` matches the regular expression `expected`.'''
    if not re.match(expected, topic):
        msg = 'Expected topic({0!r}) to match the regular expression {1!r}'.format(topic, expected)
        raise AssertionError(msg)


@assertion
def not_to_match(topic, expected):
    '''Asserts that `topic` DOES NOT match the regular expression `expected`.'''
    if re.match(expected, topic):
        msg = 'Expected topic({0!r}) not to match the regular expression {1!r}'.format(topic, expected)
        raise AssertionError(msg)



# -*- coding: utf-8 -*-

# preggy assertions
# https://github.com/heynemann/preggy

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2013 Bernardo Heynemann heynemann@gmail.com

import sys
from preggy import expect


def test_expect_not_to_be_here():
    try:
        expect.not_to_be_here()
    except AssertionError:
        err = sys.exc_info()[1]
        expect(err).to_be_an_error()
        expect(err).to_be_an_error_like(AssertionError)
        expect(err).to_have_an_error_message_of("Should not have gotten this far.")
    else:
        assert False, "Should not have gotten this far."


def test_expect_not_to_be_here_with_message():
    try:
        expect.not_to_be_here("qweqwe")
    except AssertionError:
        err = sys.exc_info()[1]
        expect(err).to_be_an_error()
        expect(err).to_be_an_error_like(AssertionError)
        expect(err).to_have_an_error_message_of("Should not have gotten this far (qweqwe).")
    else:
        assert False, "Should not have gotten this far."



# -*- coding: utf-8 -*-
'''preggy length assertions.  For use with `expect()` (see `preggy.core`).
'''


# preggy assertions
# https://github.com/heynemann/preggy

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2013 Bernardo Heynemann heynemann@gmail.com

from __future__ import absolute_import
from preggy import assertion


#-------------------------------------------------------------------------------------------------
# Helpers
#-------------------------------------------------------------------------------------------------
def _get_length(topic, expected):
    length = None
    try:
        length = len(topic)
    except (AttributeError, TypeError):
        if hasattr(topic, 'qsize'):
            length = topic.qsize()
    if length is None:
        msg = "Could not determine \"{0}\"'s length.".format(topic)
        raise AssertionError(msg)
    return length


#-------------------------------------------------------------------------------------------------
# Assertions
#-------------------------------------------------------------------------------------------------
@assertion
def to_length(topic, expected):
    '''Asserts that `len(topic)` == `expected`.'''
    length = _get_length(topic, expected)
    if length != expected:
        msg = 'Expected topic({0!r}) to have {1} of length, but it has {2}'.format(topic, expected, length)
        raise AssertionError(msg)

@assertion
def not_to_length(topic, expected):
    '''Asserts that `len(topic)` != `expected`.'''
    length = _get_length(topic, expected)
    if length == expected:
        msg = 'Expected topic({0!r}) not to have {1} of length'.format(topic, expected)
        raise AssertionError(msg)



# -*- coding: utf-8 -*-
'''preggy emptiness assertion.  For use with `expect()` (see `preggy.core`).'''


# preggy assertions
# https://github.com/heynemann/preggy

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2013 Bernardo Heynemann heynemann@gmail.com

from __future__ import absolute_import
from preggy import create_assertions


@create_assertions
def to_be_empty(topic):
    '''Asserts that the `len` of `topic` is `0`.'''
    return len(topic) == 0



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import print_function

#import os
#from os import path
#from os.path import (abspath, basename, dirname, join as joinpath)
from textwrap import dedent
import sys

try:
    from colorama import Fore, Style
except ImportError:  
    class _NoColor(object):
        def __getattribute__(self):
            return ''

    Fore = Style = _NoColor()

from preggy import __meta__
from preggy.core import _registered_assertions


#--------------------------------------------------------------------------------
#   Misc Functions
#--------------------------------------------------------------------------------
_longest_of_list= lambda lst: max([len(i) for i in lst])

def _format_module_name(m_name):
    m_formatted = m_name[18:]  # remove "assertions."

    if m_formatted.startswith('types'):
        m_formatted = m_formatted[6:]  # remove "types."

    return m_formatted.capitalize()# + ':'

def _sort_assertion(func):
    name = func.__name__
    if name.startswith('not_to_'):
        return name[7:] + '_z' # ensure that `not_...` comes second
    return name[3:]

def _map_assertions_to_modules(modules=None, assertions=None, sort_func=_sort_assertion):
    '''Returns a dict containing (module:[sorted list of assertions]).'''
    mod_to_assertions = dict.fromkeys(modules)

    for mod in mod_to_assertions.keys():
        _assertions = [f for f in assertions if mod == f.__module__]
        _assertions = sorted(_assertions, key=sort_func)
        mod_to_assertions[mod] = _assertions

    return mod_to_assertions

def _print_assertions():
    #PREGGY_DIR      = dirname(__file__)
    #ASSERTIONS_DIR  = joinpath(PREGGY_DIR, 'assertions')
    
    DIVIDER         = '-' * 55
    DIVIDER         = ''.join((Style.DIM, DIVIDER, Style.RESET_ALL))
    FMT_FIRST_LINE  = '    {colors[module]}{module:<{module_space}}{colors[reset]}{colors[assertion]}{assertions[0]}{colors[reset]}'
    FMT_OTHER_LINES = '    {indent}{colors[assertion]}{name}{colors[reset]}'

    #assertion_names = _registered_assertions.keys()
    assertion_fns   = _registered_assertions.values()

    module_names            = set(i.__module__ for i in assertion_fns)  # remove duplicates
    module_names            = sorted(module_names)
    formatted_module_names  = [_format_module_name(m) for m in module_names]

    #longest_assertion_name  = _longest_of_list(assertion_names)
    longest_module_name     = _longest_of_list(formatted_module_names)

    assertions_by_module    = _map_assertions_to_modules(
        modules=module_names,
        assertions=assertion_fns)

    for modname in module_names:
        module = _format_module_name(modname).upper()

        _assertions = [
            '{indent}{name}'.format(
                name    = f.__name__,
                indent  = '' if (f.__name__.startswith('not')) else 4*' '
            ) for f in assertions_by_module[modname] ]
        
        colors = {
            'module':   Style.DIM,
            'assertion':Style.NORMAL,
            'reset':    Fore.RESET + Style.RESET_ALL,
        }

        print(DIVIDER)
        print(FMT_FIRST_LINE.format(
                module      =module,
                module_space=longest_module_name+4,
                assertions  = _assertions,
                colors      = colors
            ))

        _assertions = [FMT_OTHER_LINES.format(
                            indent  = ' ' * (4+longest_module_name),
                            name    = name,
                            colors  = colors)
                        for name in _assertions
                    ]
        print('\n'.join(_assertions[1:]))

    print(DIVIDER)


#--------------------------------------------------------------------------------
#   The main's mane
#--------------------------------------------------------------------------------
def main():
    print('\n')
    print( Style.BRIGHT, 
         '{0:^55}'.format('Assertions Reference', ).upper(), 
         Style.RESET_ALL, 
        sep='')
    _print_assertions()
    
    INFO_STR = dedent(
    '''
    {0.__name__} {0.__version__} {Style.DIM}({0.__date__:%Y/%m/%d}){Style.RESET_ALL}
    {Fore.RESET}{Style.NORMAL}{0.__url__}{Style.RESET_ALL}{Fore.RESET}
    '''.format(__meta__, Fore=Fore, Style=Style)
    )
    
    print(INFO_STR)
    print('{0.__copyright__}'.format(__meta__), end=2*'\n')
    print('{0.__license_full__}'.format(__meta__), 
            end= 2*'\n')
    

    
if __name__ == '__main__':
    sys.exit(main())



# -*- coding: utf-8 -*-

# preggy assertions
# https://github.com/heynemann/preggy

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2014 Pablo Santiago Blum de Aguiar <scorphus@gmail.com>

from preggy import expect


def test_chaining():
    expect('qwe') \
        .not_to_be_null() \
        .to_be_instance_of(str) \
        .to_be_like('QWE') \
        .to_equal('qwe')
    expect(1729) \
        .not_to_be_null() \
        .to_be_instance_of(int) \
        .to_be_numeric() \
        .to_be_greater_or_equal_to(496) \
        .to_equal(1729)
    expect(3.14159265358979) \
        .not_to_be_null() \
        .to_be_instance_of(float) \
        .to_be_numeric() \
        .to_be_lesser_or_equal_to(3.1416) \
        .to_be_like(3.14159265358979)
    expect([1, 2, 3]) \
        .not_to_be_null() \
        .not_to_be_empty() \
        .to_be_instance_of(list) \
        .to_be_like([3, 2, 1])
    expect({'a': 'b', 'x': 'z'}) \
        .not_to_be_null() \
        .not_to_be_empty() \
        .to_be_instance_of(dict) \
        .to_be_like({'x': 'z', 'a': 'b'})

    meaning = {'x': 42}
    of_life = {'x': 42}

    expect(meaning.get('y')).to_equal(of_life.get('y'))
    expect(meaning.get('x')).not_to_be_null().to_equal(of_life.get('x'))
    try:
        expect(meaning.get('y')).not_to_be_null().to_equal(of_life.get('x'))
    except AssertionError:
        return
    else:
        assert False, 'Should not have gotten this far'



# -*- coding: utf-8 -*-
'''preggy is an assertion library for Python. (What were you ``expect()``ing?)


EXAMPLE
=======

    from preggy import expect

    def test_roses_are_red():
        rose = Rose()
        expect(rose.color).to_equal('red')

    def test_violets_are_not_red():
        violet = Violet()
        expect(violet.color).not_to_equal('red')


For more info:
http://heynemann.github.io/preggy

'''

# preggy assertions
# https://github.com/heynemann/preggy

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2013 Bernardo Heynemann heynemann@gmail.com

# flake8: noqa

from __future__ import absolute_import


from preggy.core import (assertion, create_assertions, Expect as expect)
from preggy.__meta__ import __version__
from preggy.assertions import *

__version__ = __version__



# -*- coding: utf-8 -*-

from __future__ import absolute_import  # NOQA

from preggy.assertions.emptiness import *  # NOQA
from preggy.assertions.equality import *  # NOQA
from preggy.assertions.inclusion import *  # NOQA
from preggy.assertions.length import *  # NOQA
from preggy.assertions.like import *  # NOQA
from preggy.assertions.comparison import *  # NOQA

from preggy.assertions.types import *  # NOQA



# -*- coding: utf-8 -*-

# preggy assertions
# https://github.com/heynemann/preggy

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2013 Bernardo Heynemann heynemann@gmail.com

import six
import sys

from preggy import expect


def test_is_error():
    topic = RuntimeError('Something Wrong')
    expect(topic).to_be_an_error()
    expect(topic).to_be_an_error_like(RuntimeError)
    expect(topic).to_have_an_error_message_of('Something Wrong')

    topic = ValueError('some bogus error')
    expect(topic).to_be_an_error()
    expect(topic).to_be_an_error_like(ValueError)
    expect(topic).to_have_an_error_message_of('some bogus error')


def test_not_to_be_an_error():
    NON_ERRORS = frozenset([
        0,
        2,
        tuple(),
        object(),
        'b123a',
        r'b123a',
        six.u('b123a'),
        b'b123a'
    ])

    for item in NON_ERRORS:
        expect(item).Not.to_be_an_error()
        expect(item).not_to_be_an_error()

        # try:
        #     expect(item).to_be_an_error()
        # except AssertionError as err:
        #     expect(err).to_have_an_error_message_of("Expected topic({0}) to be an error".format(item))


def test_error_messages():
    topic = Exception('1 does not equal 2')
    expect(topic).to_have_an_error_message_of('1 does not equal 2')

    try:
        expect(topic).to_have_an_error_message_of('some bogus')
    except AssertionError as err:
        e_format = "Expected topic({0!r}) to be an error with message {1!r}"
        e_values = six.text_type(topic), 'some bogus'
        e_message = e_format.format(*e_values)
        expect(err).to_have_an_error_message_of(e_message)

    try:
        expect(2).to_be_an_error()
    except AssertionError as err:
        expect(err).to_have_an_error_message_of('Expected topic(2) to be an error')


def test_to_be_an_error_like():
    try:
        expect(RuntimeError('Something Wrong')).to_be_an_error_like(ValueError)
    except AssertionError:
        return

    assert False, 'Should not have gotten this far'


def test_not_to_be_an_error_like():
    expect('Something Wrong').Not.to_be_an_error_like(ValueError)


def test_not_to_have_error_message():
    try:
        expect(RuntimeError('Something Wrong')).to_have_an_error_message_of('Something Else')
    except AssertionError:
        return

    assert False, 'Should not have gotten this far'


def test_can_trap_errors():
    err = expect.error_to_happen(RuntimeError)

    with err:
        raise RuntimeError("something is wrong")

    expect(err).to_have_an_error_message_of('something is wrong')

    with expect.error_to_happen(RuntimeError, message="something is wrong"):
        raise RuntimeError("something is wrong")


def test_can_trap_errors_unicode():
    err = expect.error_to_happen(RuntimeError)

    with err:
        raise RuntimeError(six.u("algo está errado"))

    expect(err).to_have_an_error_message_of(six.u("algo está errado"))

    with expect.error_to_happen(RuntimeError, message=six.u("algo está errado")):
        raise RuntimeError(six.u("algo está errado"))


def test_can_trap_errors_fails_if_error_does_not_happen():
    class_name = "%s.RuntimeError" % RuntimeError.__module__
    err = expect.error_to_happen(RuntimeError)

    try:
        with err:
            pass
    except AssertionError:
        error = sys.exc_info()[1]
        expect(error).to_have_an_error_message_of(
            'Expected "%s" to happen but no errors happened during execution of with block.' % class_name
        )
    else:
        expect.not_to_be_here()

    try:
        with expect.error_to_happen(RuntimeError):
            pass
    except AssertionError:
        error = sys.exc_info()[1]
        expect(error).to_have_an_error_message_of(
            'Expected "%s" to happen but no errors happened during execution of with block.' % class_name
        )
    else:
        expect.not_to_be_here()


def test_can_trap_errors_fails_if_wrong_error():
    class_name = "%s.RuntimeError" % RuntimeError.__module__
    value_class_name = "%s.ValueError" % ValueError.__module__
    err = expect.error_to_happen(RuntimeError)

    try:
        with err:
            raise ValueError("something else entirely")
    except AssertionError:
        error = sys.exc_info()[1]
        expect(error).to_have_an_error_message_of(
            'Expected "%s" to happen but "%s" happened during execution of with block.' % (class_name, value_class_name)
        )
    else:
        expect.not_to_be_here()


def test_can_trap_errors_fails_if_wrong_error_message():
    class_name = "%s.ValueError" % ValueError.__module__
    err = expect.error_to_happen(ValueError, message="Woot?")

    try:
        with err:
            raise ValueError("something else entirely")
    except AssertionError:
        error = sys.exc_info()[1]
        expect(error).to_have_an_error_message_of(
            'Expected "%s" to have a message of "Woot?", but the actual error was "something else entirely".' % class_name
        )
    else:
        expect.not_to_be_here()


def test_can_NOT_trap_errors():
    try:
        with expect.not_error_to_happen(RuntimeError):
            raise RuntimeError("something is wrong")
    except AssertionError:
        err = sys.exc_info()[1]
        expect(str(err)).to_include(
            'not to happen but it happened during execution of with block.'
        )
        return

    expect.not_to_be_here()


def test_can_trap_errors_but_fail_due_to_type():
    try:
        with expect.not_error_to_happen(RuntimeError):
            raise ValueError("something is wrong")
    except AssertionError:
        err = sys.exc_info()[1]
        expect(str(err)).to_include(
            'ValueError" happened during execution of with block.'
        )
        return

    expect.not_to_be_here()


def test_can_trap_errors_but_fail_due_to_message():
    try:
        with expect.not_error_to_happen(RuntimeError, message="qweqwe"):
            raise RuntimeError("something is wrong")
    except AssertionError:
        err = sys.exc_info()[1]
        expect(str(err)).to_include(
            'to have a message of "qweqwe", but the actual error was "something is wrong".'
        )
        return

    expect.not_to_be_here()





# -*- coding: utf-8 -*-

# preggy assertions
# https://github.com/heynemann/preggy

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2013 Bernardo Heynemann heynemann@gmail.com

import sys
from datetime import datetime
from uuid import uuid4

from preggy import expect

from tests import Comparable, AnotherComparable

#-----------------------------------------------------------------------------

STRING_TEST_DATA = b'asdqwe123'
EXPECTED_STRING_TEST_DATA = [
    'ASdQwE123',
    'ASDQWE123',
    'asdqwe123',
    '\tasd\nqwe\r123',
    '\033[0m\033[31masdqwe123\033[32m',
    '\x1b[0m\x1b[31masdqwe123\x1b[32m',
    '\x03[0m\x03[31masdqwe123\x03[32m',
    '\x1b[32;01masdqwe123\x1b[0m'
]
NOT_EXPECTED_STRING_TEST_DATA = [
    'ZASdQwE123',
    'ZASDQWE123',
    'Zasdqwe123',
]

DATETIME_TEST_DATA = datetime(2010, 11, 12, 13, 14, 15, 123456)
EXPECTED_DATETIME_TEST_DATA = [
    datetime(2010, 11, 12, 13, 14, 15, 123456),
    datetime(2010, 11, 12, 13, 14, 16, 123456),
    datetime(2010, 11, 12, 13, 14, 15, 123457)
]
NOT_EXPECTED_DATETIME_TEST_DATA = [
    datetime(2010, 11, 12, 13, 15, 15, 123456),
    datetime(2010, 11, 12, 14, 14, 15, 123456),
    datetime(2010, 11, 13, 13, 14, 15, 123456),
    datetime(2010, 12, 12, 13, 14, 15, 123456),
    datetime(2011, 11, 12, 13, 14, 15, 123456),
]

LIST_TEST_DATA = [1, 2, 3, 4, 'a', 'b', [1, 2, 3], {'a': 1, 'b': 2}]
EXPECTED_LIST_TEST_DATA = [
    [1, 2, 3, 4, 'a', {'a': 1, 'b': 2}, 'b', [1, 2, 3]],
    [4, 3, {'b': 2, 'a': 1}, 2, 1, 'b', 'a', [3, 2, 1]],
    ['b', [3, 1, 2], 2, 1, {'a': 1, 'b': 2}, 4, 3, 'a']
]
NOT_EXPECTED_LIST_TEST_DATA = [
    [1, 2, 3, 4, 'a', 'b', [2, 3], {'a': 1, 'b': 2}],
    [9, 1, 2, 3, 4, 'a', {'a': 1, 'b': 2}, 'b', [1, 2, 3]],
    [9, 4, 3, {'b': 2, 'a': 1}, 2, 1, 'b', 'a', [3, 2, 1]],
    [9, 'b', [3, 1, 2], 2, 1, {'a': 1, 'b': 2}, 4, 'a']
]

CMP_LIST_TEST_DATA = [AnotherComparable('X'), Comparable('Y'), Comparable('Z')]
EXPECTED_CMP_LIST_TEST_DATA = [
    [Comparable('Z'), Comparable('Y'), AnotherComparable('X')],
    [Comparable('Y'), Comparable('Z'), AnotherComparable('X')],
    [Comparable('Z'), AnotherComparable('X'), Comparable('Y')]
]
NOT_EXPECTED_CMP_LIST_TEST_DATA = [
    [Comparable('A'), Comparable('B'), Comparable('C')],
    [AnotherComparable('Z'), Comparable('Y'), AnotherComparable('X')],
    [Comparable('Y'), AnotherComparable('Z'), AnotherComparable('X')],
    [AnotherComparable('Z'), AnotherComparable('X'), Comparable('Y')]
]

TUPLE_TEST_DATA = (1, 2, 3, 4, 'a', 'b', (1, 2, 3), {'a': 1, 'b': 2})
EXPECTED_TUPLE_TEST_DATA = [
    (1, 2, 3, 4, 'a', {'a': 1, 'b': 2}, 'b', (1, 2, 3)),
    (4, 3, {'b': 2, 'a': 1}, 2, 1, 'b', 'a', (3, 2, 1)),
    ('b', (3, 1, 2), 2, 1, {'a': 1, 'b': 2}, 4, 3, 'a')
]
NOT_EXPECTED_TUPLE_TEST_DATA = [
    (9, 1, 2, 3, 4, 'a', {'a': 1, 'b': 2}, 'b', (1, 2, 3)),
    (9, 4, 3, {'b': 2, 'a': 1}, 2, 1, 'b', 'a', (3, 2, 1)),
    (9, 'b', (3, 1, 2), 2, 1, {'a': 1, 'b': 2}, 4, 'a')
]

SET_TEST_DATA = set([1, 2, 3, 4, 'a', 'b', (1, 2, 3)])
EXPECTED_SET_TEST_DATA = [
    set([1, 2, 3, 4, 'a', 'b', (1, 2, 3)]),
    set([4, 3, 2, 1, 'b', 'a', (3, 2, 1)]),
    set(['b', (3, 1, 2), 2, 1, 4, 3, 'a'])
]
NOT_EXPECTED_SET_TEST_DATA = [
    set([9, 1, 2, 3, 4, 'a', 'b', (1, 2, 3)]),
    set([9, 4, 3, 2, 1, 'b', 'a', (3, 2, 1)]),
    set([9, 'b', (3, 1, 2), 2, 1, 4, 'a'])
]

DICT_TEST_DATA = {'a': [1, 2, 3], 'b': {'x': 10, 'y': 20}}
EXPECTED_DICT_TEST_DATA = [
    {'a': [2, 1, 3], 'b': {'y': 20, 'x': 10}},
    {'b': {'x': 10, 'y': 20}, 'a': [2, 1, 3]}
]
NOT_EXPECTED_DICT_TEST_DATA = [
    {'Z': 'W', 'A': [2, 1, 3], 'B': {'Y': 20, 'X': 10}},
    {'Z': 'W', 'B': {'X': 10, 'Y': 20}, 'A': [2, 1, 3]}
]

CMP_DICT_TEST_DATA = {
    'a': Comparable('X'),
    'b': {'J': AnotherComparable('J'), 'K': 'K', 'L': Comparable('L')},
    'c': [1, 2, 3, Comparable('4')]
}
EXPECTED_CMP_DICT_TEST_DATA = [{
    'b': {'K': 'K', 'L': Comparable('L'), 'J': AnotherComparable('J'), },
    'c': [3, Comparable('4'), 1, 2],
    'a': Comparable('X')
}, {
    'c': [Comparable('4'), 3, 2, 1],
    'a': Comparable('X'),
    'b': {'L': Comparable('L'), 'J': AnotherComparable('J'), 'K': 'K'}
}]
NOT_EXPECTED_CMP_DICT_TEST_DATA = [{
    'a': Comparable('x'),
    'b': {'J': AnotherComparable('j'), 'K': 'k', 'L': Comparable('l')},
    'c': [5, 6, 7, Comparable('8')]
}, {
    'a': AnotherComparable('X'),
    'b': {'K': AnotherComparable('K'), 'L': 'L', 'J': Comparable('J')},
    'c': [Comparable('1'), Comparable('2'), Comparable('3'), 4]
}]

UUID_TEST_DATA = uuid4()
EXPECTED_UUID_TEST_DATA = [
    UUID_TEST_DATA
]
NOT_EXPECTED_UUID_TEST_DATA = [
    uuid4(),
    uuid4()
]

CMPCLASS_TEST_DATA = Comparable('preggy')
EXPECTED_CMPCLASS_TEST_DATA = [
    Comparable('preggy')
]
NOT_EXPECTED_CMPCLASS_TEST_DATA = [
    Comparable('PREGGY'),
    AnotherComparable('preggy')
]

#-----------------------------------------------------------------------------


def is_expected(item, expected):
    expect(item).to_be_like(expected)


def is_expected_to_fail(item, expected):
    try:
        expect(item).to_be_like(expected)
    except AssertionError:
        return

    assert False, "Shouldn't have gotten this far"


def is_not_expected(item, expected):
    expect(item).Not.to_be_like(expected)
    expect(item).not_to_be_like(expected)

#-----------------------------------------------------------------------------


def test_likeness():
    yield is_expected, None, None

    for expected_item in EXPECTED_STRING_TEST_DATA:
        yield is_expected, STRING_TEST_DATA, expected_item

    for expected_item in EXPECTED_LIST_TEST_DATA:
        yield is_expected, LIST_TEST_DATA, expected_item

    for expected_item in EXPECTED_CMP_LIST_TEST_DATA:
        yield is_expected, CMP_LIST_TEST_DATA, expected_item

    for expected_item in EXPECTED_TUPLE_TEST_DATA:
        yield is_expected, TUPLE_TEST_DATA, expected_item

    for expected_item in EXPECTED_SET_TEST_DATA:
        yield is_expected, SET_TEST_DATA, expected_item

    for expected_item in EXPECTED_DICT_TEST_DATA:
        yield is_expected, DICT_TEST_DATA, expected_item

    for expected_item in EXPECTED_CMP_DICT_TEST_DATA:
        yield is_expected, CMP_DICT_TEST_DATA, expected_item

    for expected_item in EXPECTED_DATETIME_TEST_DATA:
        yield is_expected, DATETIME_TEST_DATA, expected_item

    for expected_item in EXPECTED_UUID_TEST_DATA:
        yield is_expected, UUID_TEST_DATA, expected_item

    for expected_item in EXPECTED_CMPCLASS_TEST_DATA:
        yield is_expected, CMPCLASS_TEST_DATA, expected_item


def test_likeness_fails():
    yield is_expected, None, None

    for expected_item in NOT_EXPECTED_STRING_TEST_DATA:
        yield is_expected_to_fail, STRING_TEST_DATA, expected_item

    for expected_item in NOT_EXPECTED_LIST_TEST_DATA:
        yield is_expected_to_fail, LIST_TEST_DATA, expected_item

    for expected_item in NOT_EXPECTED_CMP_LIST_TEST_DATA:
        yield is_expected_to_fail, CMP_LIST_TEST_DATA, expected_item

    for expected_item in NOT_EXPECTED_TUPLE_TEST_DATA:
        yield is_expected_to_fail, TUPLE_TEST_DATA, expected_item

    for expected_item in NOT_EXPECTED_SET_TEST_DATA:
        yield is_expected_to_fail, SET_TEST_DATA, expected_item

    for expected_item in NOT_EXPECTED_DICT_TEST_DATA:
        yield is_expected_to_fail, DICT_TEST_DATA, expected_item

    for expected_item in NOT_EXPECTED_CMP_DICT_TEST_DATA:
        yield is_expected_to_fail, CMP_DICT_TEST_DATA, expected_item

    for expected_item in NOT_EXPECTED_DATETIME_TEST_DATA:
        yield is_expected_to_fail, DATETIME_TEST_DATA, expected_item

    for expected_item in NOT_EXPECTED_UUID_TEST_DATA:
        yield is_expected_to_fail, UUID_TEST_DATA, expected_item

    for expected_item in NOT_EXPECTED_CMPCLASS_TEST_DATA:
        yield is_expected_to_fail, CMPCLASS_TEST_DATA, expected_item


def test_not_likeness():
    yield is_not_expected, None, 1

    for not_expected_item in NOT_EXPECTED_STRING_TEST_DATA:
        yield is_not_expected, STRING_TEST_DATA, not_expected_item

    for not_expected_item in NOT_EXPECTED_LIST_TEST_DATA:
        yield is_not_expected, LIST_TEST_DATA, not_expected_item

    for not_expected_item in NOT_EXPECTED_CMP_LIST_TEST_DATA:
        yield is_not_expected, CMP_LIST_TEST_DATA, not_expected_item

    for not_expected_item in NOT_EXPECTED_TUPLE_TEST_DATA:
        yield is_not_expected, TUPLE_TEST_DATA, not_expected_item

    for not_expected_item in NOT_EXPECTED_SET_TEST_DATA:
        yield is_not_expected, SET_TEST_DATA, not_expected_item

    for not_expected_item in NOT_EXPECTED_DICT_TEST_DATA:
        yield is_not_expected, DICT_TEST_DATA, not_expected_item

    for not_expected_item in NOT_EXPECTED_CMP_DICT_TEST_DATA:
        yield is_not_expected, CMP_DICT_TEST_DATA, not_expected_item

    for expected_item in NOT_EXPECTED_DATETIME_TEST_DATA:
        yield is_not_expected, DATETIME_TEST_DATA, expected_item

    for expected_item in NOT_EXPECTED_UUID_TEST_DATA:
        yield is_not_expected, UUID_TEST_DATA, expected_item

    for expected_item in NOT_EXPECTED_CMPCLASS_TEST_DATA:
        yield is_not_expected, CMPCLASS_TEST_DATA, expected_item


def test_likeness_of_objects():
    try:
        expect(object()).to_be_like(1)
    except RuntimeError:
        err = sys.exc_info()[1]
        assert err.__class__ is RuntimeError


def test_likeness_of_different_types():
    expect(1).not_to_be_like('1')



# -*- coding: utf-8 -*-
'''preggy comparison assertion.  For use with `expect()` (see `preggy.core`).
'''


# preggy assertions
# https://github.com/heynemann/preggy

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2013 Bernardo Heynemann heynemann@gmail.com

from __future__ import absolute_import

from preggy import create_assertions
from preggy import utils


@create_assertions
def to_be_greater_than(topic, expected):
    '''Asserts that `topic > expected`.'''
    topic = utils.fix_string(topic)
    expected = utils.fix_string(expected)

    if isinstance(expected, (tuple, list, set, dict)):
        return len(topic) > len(expected)

    return topic > expected


@create_assertions
def to_be_lesser_than(topic, expected):
    '''Asserts that `topic < expected`.'''
    topic = utils.fix_string(topic)
    expected = utils.fix_string(expected)

    if isinstance(expected, (tuple, list, set, dict)):
        return len(topic) < len(expected)

    return topic < expected


@create_assertions
def to_be_greater_or_equal_to(topic, expected):
    '''Asserts that `topic >= expected`.'''
    topic = utils.fix_string(topic)
    expected = utils.fix_string(expected)

    if isinstance(expected, (tuple, list, set, dict)):
        return len(topic) >= len(expected)

    return topic >= expected


@create_assertions
def to_be_lesser_or_equal_to(topic, expected):
    '''Asserts that `topic <= expected`.'''
    topic = utils.fix_string(topic)
    expected = utils.fix_string(expected)

    if isinstance(expected, (tuple, list, set, dict)):
        return len(topic) <= len(expected)

    return topic <= expected



# -*- coding: utf-8 -*-

# preggy assertions
# https://github.com/heynemann/preggy

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2013 Bernardo Heynemann heynemann@gmail.com

from preggy import expect

#-----------------------------------------------------------------------------

def test_to_match():
    expect('a123b').to_match(r'a\d+b')

    try:
        expect('a123b').not_to_match(r'a\d+b')
    except AssertionError:
        return

    assert False, 'Should not have gotten this far'


def test_not_to_match():
    expect('b123a').Not.to_match(r'a\d+b')
    expect('b123a').not_to_match(r'a\d+b')

    try:
        expect('b123a').to_match(r'a\d+b')
    except AssertionError:
        return

    assert False, 'Should not have gotten this far'



# -*- coding: utf-8 -*-
'''preggy function assertions.  For use with `expect()` (see `preggy.core`).
'''

# preggy assertions
# https://github.com/heynemann/preggy

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2013 Bernardo Heynemann heynemann@gmail.com


from __future__ import absolute_import
import inspect

from preggy import assertion

#-------------------------------------------------------------------------------------------------
# Helpers
#-------------------------------------------------------------------------------------------------
_is_func = lambda topic: inspect.ismethod(topic) or inspect.isfunction(topic)


#-------------------------------------------------------------------------------------------------
# Assertions
#-------------------------------------------------------------------------------------------------
@assertion
def to_be_a_function(topic):
    '''Asserts that `topic` is a function.'''
    if not _is_func(topic):
        msg = 'Expected topic({0}) to be a function or a method, but it was a {1}'.format(
            topic,
            topic.__class__
            )
        raise AssertionError(msg)


@assertion
def not_to_be_a_function(topic):
    '''Asserts that `topic` is NOT a function.'''
    if _is_func(topic):
        msg = 'Expected topic({0}) not to be a function or a method'.format(topic)
        raise AssertionError(msg)



# -*- coding: utf-8 -*-

# preggy assertions
# https://github.com/heynemann/preggy

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2013 Bernardo Heynemann heynemann@gmail.com

from preggy import expect

#-----------------------------------------------------------------------------

def fake_method(): 
    pass
class FakeClass(object):
    def whatever(self): 
        pass

TEST_DATA = frozenset([
    fake_method,
    FakeClass.whatever,
    FakeClass().whatever
])

#-----------------------------------------------------------------------------

def is_expected(item):
    expect(item).to_be_a_function()
    try:
        expect(item).not_to_be_a_function()
    except AssertionError:
        return
    assert False, 'Should not have gotten this far'


def is_not_expected(item):
    expect(item).Not.to_be_a_function()
    expect(item).not_to_be_a_function()

    try:
        expect(item).to_be_a_function()
    except AssertionError:
        return
    assert False, 'Should not have gotten this far'

#-----------------------------------------------------------------------------

def test_to_be_a_function():
    for item in TEST_DATA:
        yield is_expected, item


def test_not_to_be_a_function():
    is_not_expected('a')
    is_not_expected((1, 2))
    is_not_expected([1, 2])



# -*- coding: utf-8 -*-
'''preggy boolean assertions.  For use with `expect()` (see `preggy.core`).
'''

# preggy assertions
# https://github.com/heynemann/preggy

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2013 Bernardo Heynemann heynemann@gmail.com

from __future__ import absolute_import
from preggy import assertion


@assertion
def to_be_true(topic):
    '''Asserts that the boolean value of `topic` == `True`.'''
    if not bool(topic):
        msg = 'Expected topic({0}) to be truthy'.format(topic)
        raise AssertionError(msg)


@assertion
def to_be_false(topic):
    '''Asserts that the boolean value of `topic` == `False`.'''
    if bool(topic):
        msg = 'Expected topic({0}) to be falsy'.format(topic)
        raise AssertionError(msg)


@assertion
def not_to_be_true(topic):
    '''Asserts that the boolean value of `topic` != `True`.'''
    return to_be_false(topic)


@assertion
def not_to_be_false(topic):
    '''Asserts that the boolean value of `topic` != `False`.'''
    return to_be_true(topic)



# -*- coding: utf-8 -*-
'''preggy 'like' assertions.  For use with `expect()` (see `preggy.core`).
'''


# preggy assertions
# https://github.com/heynemann/preggy

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2013 Bernardo Heynemann heynemann@gmail.com
from __future__ import absolute_import, print_function
import re
from datetime import datetime
import difflib
from uuid import UUID

try:
    from six import string_types, binary_type, PY2, PY3
except ImportError:  # pragma: no cover
    import warnings
    warnings.warn('Ignoring six. Probably setup.py installing package.')

import numbers

from preggy import assertion
from preggy import utils

__all__ = ('to_be_like', 'not_to_be_like')

#-------------------------------------------------------------------------------------------------
#   CONSTANTS
#-------------------------------------------------------------------------------------------------
DATE_THRESHOLD = 5.0

RESET = '\033[m'
RED = '\033[31m'
GREEN = '\033[32m'
YELLOW = '\033[33m'

REMOVE_COLORS_REGEX = re.compile(
    r'(\033|\x1b|\x03)'  # prefixes
    r'\['                # non-regex bracket
    r'([0-9]*[;])?'      # semi-colon
    r'[0-9]*m',          # suffix
    flags=re.UNICODE
)

NORMALIZE_WHITESPACE_REGEX = re.compile(
    r'\s+',
    flags=re.UNICODE | re.MULTILINE | re.IGNORECASE
)

#-------------------------------------------------------------------------------------------------
#   HELPERS
#-------------------------------------------------------------------------------------------------
_filter_str = lambda s: NORMALIZE_WHITESPACE_REGEX.sub('', s.lower()).strip()


def compare(first, second):
    matcher = difflib.SequenceMatcher(None, first, second)
    first = get_match_for_text(matcher, first, True)
    second = get_match_for_text(matcher, second, True)
    return matcher, first, second


def get_match_for_text(matcher, text, first):
    result = []
    COLOR_MAP = {
        'delete': RED,
        'insert': GREEN,
        'replace': YELLOW
    }

    for tag, i1, i2, j1, j2 in matcher.get_opcodes():
        start, stop = (i1, i2) if first else (j1, j2)
        to_append = text[start:stop]
        if tag in COLOR_MAP:
            to_append = ''.join((COLOR_MAP[tag], to_append, RESET))
        result.append(to_append)

    return ''.join(result)


def _match_alike(expected, topic, diff=False):
    '''Determines the types of `expected` and `topic`, and calls the appropriate comparison function.'''
    if topic is None:
        return expected is None

    if isinstance(topic, UUID):
        return _compare_uuid(expected, topic)
    if isinstance(topic, string_types + (binary_type, )):
        return _compare_strings(expected, topic)
    if isinstance(topic, numbers.Number):
        return _compare_numbers(expected, topic)
    if isinstance(topic, (list, tuple, set)):
        return _compare_lists(expected, topic)
    if isinstance(topic, dict):
        return _compare_dicts(expected, topic)
    if isinstance(topic, datetime):
        return _compare_datetime(expected, topic)
    if _is_comparable(topic) and _is_comparable(expected):
        return _compare_comparables(expected, topic)

    raise RuntimeError('Could not compare {expected} and {topic}'.format(expected=expected, topic=topic))


def _strip_string(text):
    if not text:
        return text
    
    text = utils.fix_string(text)
    text = REMOVE_COLORS_REGEX.sub('', text)
    text = _filter_str(text)
    
    return text


def _is_comparable(obj):
    if PY2:
        return hasattr(obj, '__eq__') or hasattr(obj, '__cmp__')
    if PY3:
        return obj.__class__.__eq__ != object.__eq__


def _compare_strings(expected, topic):
    '''Asserts the "like"-ness of `topic` and `expected` as strings.
    Allows some leeway.  (Strings don't have to exactly match.)

    '''
    topic = _strip_string(topic)
    expected = _strip_string(expected)
    return expected == _filter_str(topic)


def _compare_uuid(expected, topic):
    '''Asserts the "like"-ness of `topic` and `expected` as UUID.'''
    topic = str(topic)
    expected = str(expected)
    return expected == topic


def __timedelta_to_seconds(timedelta):
    ms = 10 ** 6          # microseconds/second
    days = 24 * 60 * 60     # seconds/day

    ms_as_seconds = float(timedelta.microseconds) / ms
    seconds = float(timedelta.seconds)
    days_as_seconds = float(timedelta.days) * days
    total_seconds = sum((ms_as_seconds,
                         seconds,
                         days_as_seconds))
    return abs(total_seconds)  # abs() comes last


def _compare_datetime(expected, topic):
    return __timedelta_to_seconds(topic - expected) <= DATE_THRESHOLD


def _compare_numbers(expected, topic):
    '''Asserts the "like"-ness of `topic` and `expected` as Numbers.'''
    FALSE_CONDITIONS = (not isinstance(topic, numbers.Number),
                        not isinstance(expected, numbers.Number), )
    if any(FALSE_CONDITIONS):
        return False
    return float(expected) == float(topic)


def _compare_dicts(expected, topic):
    '''Asserts the "like"-ness of `topic` and `expected` as dicts.'''
    return _match_dicts(expected, topic) and _match_dicts(topic, expected)


def _match_dicts(expected, topic):
    '''Asserts the "like"-ness of all keys and values in `topic` and `expected`.'''
    for k, v in expected.items():
        if not k in topic or not _match_alike(topic[k], v):
            return False
    return True


def _compare_lists(expected, topic):
    '''Asserts the "like"-ness of `topic` and `expected` as lists.'''
    return _match_lists(expected, topic) and _match_lists(topic, expected)


def _compare_comparables(expected, topic):
    '''Asserts the "like"-ness of `topic` and `expected` as comparable objects.'''
    try:
        return expected == topic
    except:
        return False


def _match_lists(expected, topic):
    '''Asserts the "like"-ness each item in of `topic` and `expected` (as lists or tuples).'''
    # TODO: Rewrite this using itertools
    #       http://docs.python.org/2/library/itertools.html
    for item in expected:
        if isinstance(item, (list, tuple)):
            found = False
            for inner_item in topic:
                if isinstance(inner_item, (list, tuple)) and _compare_lists(item, inner_item):
                    found = True
                    break
            if not found:
                return False
        else:
            found = False
            for t in topic:
                try:
                    if item == t:
                        found = True
                        break
                except:
                    continue
            if not found:
                return False
    return True


#-------------------------------------------------------------------------------------------------
# Assertions
#-------------------------------------------------------------------------------------------------
@assertion
def to_be_like(topic, expected, diff=True):
    '''Asserts that `topic` is like (similar to) `expected`. Allows some leeway.'''
    result = _match_alike(expected, topic, diff=diff)
    is_str = lambda x: isinstance(x, string_types + (binary_type,))
    if not result:
        if diff is True and (
            is_str(topic) and
            is_str(expected)
        ):
            first, second = _strip_string(topic), _strip_string(expected)
            matcher, first, second = compare(first, second)
            print()
            print('Expected strings to be equal, but they were different:')
            print(first)
            print(second)
            print()
        raise AssertionError("Expected topic('{topic}') to be like '{expected}'".format(topic=topic, expected=expected))


@assertion
def not_to_be_like(topic, expected, diff=False):
    '''Asserts that `topic` is NOT like (NOT similar to) `expected`. Allows some leeway.'''
    result = _match_alike(expected, topic, diff=diff)
    if result:
        raise AssertionError("Expected topic('{topic}') not to be like '{expected}'".format(topic=topic, expected=expected))



# -*- coding: utf-8 -*-
'''preggy equality assertion.  For use with `expect()` (see `preggy.core`).
'''


# preggy assertions
# https://github.com/heynemann/preggy

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2013 Bernardo Heynemann heynemann@gmail.com

from __future__ import absolute_import

from preggy import create_assertions
from preggy import utils


@create_assertions
def to_equal(topic, expected):
    '''Asserts that `topic == expected`.'''
    topic = utils.fix_string(topic)
    expected = utils.fix_string(expected)

    try:
        return expected == topic
    except:
        return False



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# preggy assertions
# https://github.com/heynemann/preggy

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2011 Bernardo Heynemann heynemann@gmail.com

from setuptools import setup, find_packages

from preggy import __doc__, __meta__, __version__

REQUIREMENTS = {
    'install': ['six', 'unidecode'],
    'extras': {
        'tests':['nose', 'yanc', 'coverage', 'tox',]
    }
}


setup(
    name        ='preggy',
    version     =__version__,
    description =__doc__.splitlines()[0],
    long_description=__doc__,
    keywords    =__meta__.__keywords__,
    
    author      =__meta__.__author__,
    author_email=__meta__.__author_email__,
    
    maintainer  =__meta__.__maintainer__,
    maintainer_email=__meta__.__maintainer_email__,
    
    url         =__meta__.__url__,
    download_url=__meta__.__download_url__,

    ### For future, when Python packaging gets its crap together. See:
    ###   http://stackoverflow.com/questions/14459828/how-to-set-bug-tracker-url-in-setup-py-script
    #bugtrack_url='http://github.com/heynemann/preggy/issues',

    license     =__meta__.__license__,
    
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: MacOS',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: POSIX',
        'Operating System :: Unix',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Testing'
    ],

    packages=find_packages(exclude=['tests', 'tests.*']),
    
    install_requires= REQUIREMENTS['install'],
    extras_require  = REQUIREMENTS['extras'],

    entry_points={},
)



# -*- coding: utf-8 -*-
'''Assorted helpers used elsewhere in preggy.

Currently contains only string formatting code, but this may (or may not) change.
'''
# preggy assertions
# https://github.com/heynemann/preggy

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2013 Bernardo Heynemann heynemann@gmail.com

from __future__ import absolute_import
import logging
import re

try:
    import six
    text_type = six.text_type
except ImportError:  # pragma: no cover
    import warnings
    warnings.warn('Ignoring six. Probably setup.py installing package.')

try:
    from unidecode import unidecode
except ImportError:  # pragma: no cover
    import warnings
    warnings.warn('Ignoring unidecode. Probably setup.py installing package.')


logging.basicConfig(level=logging.INFO)
log = logging.getLogger(__name__)

UNDERSCORES = re.compile(r'_+')

def humanized_name(thing):
    try:                    return UNDERSCORES.sub(' ', thing.__name__)
    except AttributeError:  return UNDERSCORES.sub(' ', thing)

def format_assertion_msg(assertion_clause, *args):
    raw_msg = 'Expected topic({{0!r}}) {assertion_clause}'
    raw_msg = raw_msg.format(assertion_clause=assertion_clause)
    if len(args) is 2:
        raw_msg += ' {1!r}'
    return raw_msg

def fix_string(obj):
    if isinstance(obj, (six.binary_type, )):
        try:
            return obj.decode('utf-8')
        except Exception:
            return obj
    return obj


class AssertionsMap(dict):
    '''A simple dict with a dash of logging.'''

    def __getitem__(self, k):
        # log.debug('fetching assertion: {name!r}'.format(name=k))
        return super(AssertionsMap, self).__getitem__(k)

    def __setitem__(self, k, v):
        _keys = self.keys()
        # commented due to noise in tests
        # if k not in _keys:
            # log.debug('registering assertion: {name!r}, with function: {func!r}'.format(name=k, func=v))
        # elif self[k] is not v:  # existing key with new assertion function
            # log.debug('re-registering assertion: {name!r}, with new function: {func!r}'.format(name=k, func=v))
        # elif self[k] is v:  # same key, same function
            # return
        if k in _keys and self[k] is v:  # same key, same function
            return
        return super(AssertionsMap, self).__setitem__(k, v)

    def __delitem__(self, k):
        # log.debug('deleting assertion: {name!r}'.format(name=k))
        return super(AssertionsMap, self).__delitem__(k)



# -*- coding: utf-8 -*-

# preggy assertions
# https://github.com/heynemann/preggy

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2013 Bernardo Heynemann heynemann@gmail.com

from preggy import expect

from tests import Comparable

#-----------------------------------------------------------------------------

TEST_DATA = (
    'my_string',
    [1, 2, 3, ],
    {'a': 1, 'b': 2},
    tuple([1, 2, 3]),
    '123',
    '3.14',
    (None, Comparable(), 'deliberately the last one')
)

INCLUDED_DATA = (
    'str',
    1,
    'a',
    2,
    1,
    3.1,
    'deliberately the last one'
)

NOT_INCLUDED_DATA = (
    'potatoh',
    4,
    'c',
    5,
    4,
    4.2,
    2.718281828
)

#-----------------------------------------------------------------------------


def is_included(item, expected):
    expect(item).to_include(expected)


def is_not_included(item, expected):
    expect(item).Not.to_include(expected)
    expect(item).not_to_include(expected)

#-----------------------------------------------------------------------------


def test_includes():
    for index, item in enumerate(TEST_DATA):
        yield is_included, item, INCLUDED_DATA[index]


def test_not_includes():
    for index, item in enumerate(TEST_DATA):
        yield is_not_included, item, NOT_INCLUDED_DATA[index]
